import 'package:flutter/material.dart';
import 'package:hub/features/backend/index.dart';
import 'package:hub/features/storage/index.dart';
import 'package:hub/flutter_flow/flutter_flow_model.dart';
import 'package:hub/flutter_flow/request_manager.dart';
import 'package:hub/pages/message_history_page/message_history_page_widget.dart';

class MessageHistoryPageModel
    extends FlutterFlowModel<MessageHistoryPageWidget> {
  late final String devUUID;
  late final String userUUID;
  late final String cliUUID;

  final unfocusNode = FocusNode();
  FocusNode? textFieldFocusNode;
  TextEditingController? textController;
  String? Function(BuildContext, String?)? textControllerValidator;

  TabController? tabBarController;
  int get tabBarCurrentIndex =>
      tabBarController != null ? tabBarController!.index : 0;

  final _getLiberationsManager = StreamRequestManager<ApiCallResponse>();
  Stream<ApiCallResponse> getLiberations({
    String? uniqueQueryKey,
    bool? overrideCache,
    required Stream<ApiCallResponse> Function() requestFn,
  }) =>
      _getLiberationsManager.performRequest(
        uniqueQueryKey: uniqueQueryKey,
        overrideCache: overrideCache,
        requestFn: requestFn,
      );
  void clearGetLiberationsCache() => _getLiberationsManager.clear();
  void clearGetLiberationsCacheKey(String? uniqueKey) =>
      _getLiberationsManager.clearRequest(uniqueKey);

  @override
  void initState(BuildContext context) {
    init();
  }

  Future<void> init() async {
    devUUID =
        (await StorageRepositoryImpl().get(ProfileStorageKey.devUUID.key)) ??
            '';
    userUUID =
        (await StorageRepositoryImpl().get(ProfileStorageKey.userUUID.key)) ??
            '';
    cliUUID =
        (await StorageRepositoryImpl().get(ProfileStorageKey.clientUUID.key)) ??
            '';
  }

  @override
  void dispose() {
    unfocusNode.dispose();
    textFieldFocusNode?.dispose();
    textController?.dispose();
    tabBarController?.dispose();

    clearGetLiberationsCache();
  }
}
